@@ -80,7 +80,7 @@ int ehd_load(const struct ehd_mtreq *req, struct ehd_mount *mt)
 		}
 	}
 
-#if defined(__linux__)
+#ifdef HAVE_LIBCRYPTSETUP
 	ret = ehd_dmcrypt_ops.load(req, mt);
 #elif defined(HAVE_DEV_CGDVAR_H)
 	ret = ehd_cgd_ops.load(req, mt);
@@ -115,7 +115,7 @@ int ehd_unload(const struct ehd_mount *mt)
 {
 	int ret;
 
-#if defined(__linux__)
+#ifdef HAVE_LIBCRYPTSETUP
 	ret = ehd_dmcrypt_ops.unload(mt);
 #elif defined(HAVE_DEV_CGDVAR_H)
 	ret = ehd_cgd_ops.unload(mt);
