@@ -103,9 +103,9 @@ void arglist_log(const struct HXdeque *argq)
 
 	str = HXmc_meminit(NULL, 80);
 	for (n = argq->first; n != NULL; n = n->next) {
-		HXmc_strcat(&str, "[");
+		HXmc_strcat(&str, "'");
 		HXmc_strcat(&str, n->ptr);
-		HXmc_strcat(&str, "] ");
+		HXmc_strcat(&str, "' ");
 	}
 
 	misc_warn("command: %s\n", str);
@@ -121,9 +121,9 @@ void arglist_llog(const char *const *argv)
 
 	str = HXmc_meminit(NULL, 80);
 	while (*argv != NULL) {
-		HXmc_strcat(&str, "[");
+		HXmc_strcat(&str, "'");
 		HXmc_strcat(&str, *argv);
-		HXmc_strcat(&str, "] ");
+		HXmc_strcat(&str, "' ");
 		++argv;
 	}
 
@@ -144,7 +144,7 @@ void arglist_add(struct HXdeque *argq, const char *arg,
 {
 	char *filled;
 
-	if (HXformat_aprintf(vinfo, &filled, arg) == 0)
+	if (HXformat2_aprintf(vinfo, &filled, arg) == 0)
 		/*
 		 * This case may happen with e.g. %(before="-o" OPTIONS) where
 		 * OPTIONS is empty. And options expanding to nothing are
