@@ -85,7 +85,7 @@ static void run_ofl(const struct config *const config,
 	hxmc_t *mntpt = NULL;
 	struct stat sb;
 
-	HXformat_aprintf(vinfo, &mntpt, "%(MNTPT)");
+	HXformat2_aprintf(vinfo, &mntpt, "%(MNTPT)");
 	if (!(stat(mntpt, &sb) < 0 && errno == ENOENT)) {
 		ofl_printf = misc_warn;
 		ofl(mntpt, 0);
@@ -237,7 +237,10 @@ static bool mkmountpoint_real(struct vol *const volume, const char *const d)
 		goto out;
 	}
 	if (chown(d, passwd_ent->pw_uid, passwd_ent->pw_gid) < 0) {
-		l0g("could not chown %s to %s\n", d, volume->user);
+		l0g("could not chown %s to %s(%u:%u): %s\n", d, volume->user,
+		    static_cast(unsigned int, passwd_ent->pw_uid),
+		    static_cast(unsigned int, passwd_ent->pw_gid),
+		    strerror(errno));
 		ret = false;
 		goto out;
 	}
@@ -446,7 +449,7 @@ static void mount_set_fsck(const struct config *config,
 	string = HXmc_meminit(NULL, 0);
 
 	for (i = config->command[CMD_FSCK]->first; i != NULL; i = i->next) {
-		if (HXformat_aprintf(vinfo, &current, i->ptr) > 0) {
+		if (HXformat2_aprintf(vinfo, &current, i->ptr) > 0) {
 			HXmc_strcat(&string, current);
 			HXmc_strcat(&string, " ");
 		}
@@ -678,6 +681,10 @@ void umount_final(struct config *config)
 {
 	struct vol *vol;
 
+	if (HXlist_empty(&config->volume_list.list))
+		/* Avoid needlessy waiting on usleep */
+		return;
+
 	if (config->sig_hup)
 		HXlist_for_each_entry_rev(vol, &config->volume_list, list)
 			ofl(vol->mountpoint, SIGHUP);
