@@ -1,5 +1,5 @@
 /*
- *	Copyright © Jan Engelhardt, 2008 - 2009
+ *	Copyright © Jan Engelhardt, 2008 - 2010
  *
  *	This file is part of pam_mount; you can redistribute it and/or
  *	modify it under the terms of the GNU Lesser General Public License
@@ -12,17 +12,14 @@
 #include <sys/wait.h>
 #include <assert.h>
 #include <errno.h>
+#include <limits.h>
 #include <signal.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <libHX/ctype_helper.h>
-#include <libHX/defs.h>
-#include <libHX/option.h>
-#include <libHX/proc.h>
-#include <libHX/string.h>
+#include <libHX.h>
 #include <openssl/evp.h>
 #include "pam_mount.h"
 
@@ -157,7 +154,7 @@ static void mtcr_parse_suboptions(const struct HXoptcb *cbi)
 		else if (strcmp(key, "keyfile") == 0)
 			mo->fsk_file = value;
 		else if (strcmp(key, "keysize") == 0)
-			mo->trunc_keysize = strtoul(value, NULL, 0);
+			mo->trunc_keysize = strtoul(value, NULL, 0) / CHAR_BIT;
 		else if (strcmp(key, "fsck") == 0)
 			mo->fsck = true;
 		else if (strcmp(key, "loop") == 0)
@@ -203,6 +200,14 @@ static void mtcr_parse_suboptions(const struct HXoptcb *cbi)
 	}
 }
 
+/**
+ * keyfile passthru (kfpt)
+ */
+static bool kfpt_selected(const char *c)
+{
+	return c != NULL && strcmp(c, "none") == 0;
+}
+
 static bool mtcr_get_mount_options(int *argc, const char ***argv,
     struct mount_options *opt)
 {
@@ -219,6 +224,8 @@ static bool mtcr_get_mount_options(int *argc, const char ***argv,
 		HXOPT_AUTOHELP,
 		HXOPT_TABLEEND,
 	};
+	bool kfpt;
+	int ret;
 
 	if (HX_getopt(options_table, argc, argv, HXOPT_USAGEONERR) <= 0)
 		return false;
@@ -288,40 +295,79 @@ static bool mtcr_get_mount_options(int *argc, const char ***argv,
 		return false;
 	}
 
-	if (opt->fsk_file != NULL) {
+	kfpt = kfpt_selected(opt->fsk_cipher);
+	if (opt->fsk_file == NULL) {
+		if (opt->fsk_cipher != NULL)
+			fprintf(stderr, "%s: fsk_cipher is ignored because no "
+			        "keyfile given\n", **argv);
+		if (opt->fsk_hash != NULL)
+			fprintf(stderr, "%s: fsk_hash is ignored because no "
+			        "keyfile given\n", **argv);
+	} else {
 		if (opt->fsk_cipher == NULL) {
 			fprintf(stderr, "%s: No openssl cipher specified "
 			        "(use -o fsk_cipher=xxx)\n", **argv);
 			return false;
-		} else if (opt->fsk_hash == NULL) {
+		} else if (!kfpt && opt->fsk_hash == NULL) {
 			fprintf(stderr, "%s: No openssl hash specified "
 			        "(use -o fsk_hash=xxx)\n", **argv);
 			return false;
 		}
 	}
 
-#ifdef __linux__
-	if (!dmc_is_luks(opt->container, opt->blkdev) &&
-	    opt->dmcrypt_cipher == NULL) {
-		fprintf(stderr, "%s: No dmcrypt cipher specified "
-		        "(use -o cipher=xxx)\n", **argv);
-		return false;
+#ifdef HAVE_LIBCRYPTSETUP
+	ret = dmc_is_luks(opt->container, opt->blkdev);
+	if (ret > 0) {
+		/* LUKS */
+		if (opt->dmcrypt_cipher != NULL)
+			fprintf(stderr, "%s: dmcrypt cipher ignored for LUKS "
+			        "volumes\n", **argv);
+		if (opt->dmcrypt_hash != NULL)
+			fprintf(stderr, "%s: dmcrypt hash ignored for LUKS "
+			        "volumes\n", **argv);
+	} else if (ret == 0) {
+		/* PLAIN */
+		if (opt->dmcrypt_cipher == NULL) {
+			fprintf(stderr, "%s: No dmcrypt cipher specified "
+			        "(use -o cipher=xxx)\n", **argv);
+			return false;
+		}
 	}
 #endif
 
 	if (opt->dmcrypt_hash == NULL)
 		opt->dmcrypt_hash = "plain";
+	if (!kfpt)
+		opt->fsk_password = pmt_get_password(NULL);
+	return true;
+}
 
-	if (opt->dmcrypt_hash != NULL &&
-	    strcmp(opt->dmcrypt_hash, "plain") == 0 &&
-	    opt->trunc_keysize != 0) {
-		fprintf(stderr, "%s: keysize option ignored in plain mode "
-		        "(automatically derived from filesize)\n", **argv);
-		opt->trunc_keysize = 0;
+static hxmc_t *mtcr_slurp_file(const char *file)
+{
+	hxmc_t *buf, *ln = NULL;
+	struct stat sb;
+	FILE *fp;
+
+	fp = fopen(file, "r");
+	if (fp == NULL) {
+		fprintf(stderr, "Could not open %s: %s\n",
+		        file, strerror(errno));
+		return NULL;
 	}
 
-	opt->fsk_password = pmt_get_password(NULL);
-	return true;
+	if (fstat(fileno(fp), &sb) == 0 && S_ISREG(sb.st_mode))
+		buf = HXmc_meminit(NULL, sb.st_size);
+	else
+		buf = HXmc_meminit(NULL, 4096 / CHAR_BIT);
+
+	if (buf == NULL) {
+		fprintf(stderr, "%s\n", strerror(errno));
+	} else {
+		while (HX_getl(&ln, fp) != NULL)
+			HXmc_strcat(&buf, ln);
+	}
+	fclose(fp);
+	return buf;
 }
 
 /**
@@ -342,6 +388,9 @@ static int mtcr_mount(struct mount_options *opt)
 		.fs_cipher = opt->dmcrypt_cipher,
 		.fs_hash   = opt->dmcrypt_hash,
 		.readonly  = opt->readonly,
+
+		/* Hack for CRYPT_PLAIN. */
+		.trunc_keysize = 256 / CHAR_BIT,
 	};
 
 	if (opt->fsk_file == NULL) {
@@ -352,6 +401,12 @@ static int mtcr_mount(struct mount_options *opt)
 			fprintf(stderr, "HXmc_dup: %s\n", strerror(errno));
 			return 0;
 		}
+		/* Leave trunc_keysize at 0 */
+	} else if (kfpt_selected(opt->fsk_cipher)) {
+		key = mtcr_slurp_file(opt->fsk_file);
+		if (key == NULL)
+			return 0;
+		mount_request.trunc_keysize = HXmc_length(key);
 	} else {
 		key = ehd_decrypt_key(opt->fsk_file, opt->fsk_hash,
 		      opt->fsk_cipher, opt->fsk_password);
@@ -359,11 +414,13 @@ static int mtcr_mount(struct mount_options *opt)
 			fprintf(stderr, "Error while decrypting fskey\n");
 			return 0;
 		}
+		mount_request.trunc_keysize = HXmc_length(key);
 	}
 
 	mount_request.key_data = key;
 	mount_request.key_size = HXmc_length(key);
-	mount_request.trunc_keysize = opt->trunc_keysize;
+	if (opt->trunc_keysize != 0)
+		mount_request.trunc_keysize = opt->trunc_keysize;
 
 	if ((ret = ehd_load(&mount_request, &mount_info)) < 0) {
 		fprintf(stderr, "ehd_load: %s\n", strerror(errno));
@@ -439,7 +496,6 @@ static int mtcr_mount(struct mount_options *opt)
 	    mount_info.crypto_device)) <= 0) {
 		fprintf(stderr, "pmt_cmtab_add: %s\n", strerror(errno));
 		/* ignore error on cmtab - let user have his crypto */
-		ret = 0;
 	} else if (opt->no_update) {
 		/* awesome logic */;
 	} else {
@@ -577,7 +633,8 @@ static int mtcr_umount(struct umount_options *opt)
 		final_ret = 0;
 		ehd_unload(&mount_info);
 	} else if ((ret = ehd_unload(&mount_info)) <= 0) {
-		final_ret = ret;
+		fprintf(stderr, "ehd_unload: %s\n", strerror(-ret));
+		final_ret = 0;
 	} else {
 		if (!opt->no_update)
 			pmt_smtab_remove(mountpoint, SMTABF_MOUNTPOINT);
@@ -590,6 +647,14 @@ static int mtcr_umount(struct umount_options *opt)
 
 int main(int argc, const char **argv)
 {
+	int ret;
+
+	ret = HX_init();
+	if (ret <= 0) {
+		fprintf(stderr, "HX_init: %s\n", strerror(errno));
+		abort();
+	}
+
 	Debug = false;
 	pmtlog_path[PMTLOG_ERR][PMTLOG_STDERR] = true;
 	pmtlog_prefix = HX_basename(*argv);
@@ -598,7 +663,12 @@ int main(int argc, const char **argv)
 	OpenSSL_add_all_ciphers();
 	OpenSSL_add_all_digests();
 
-	if (strncmp(HX_basename(*argv), "umount", strlen("umount")) == 0) {
+	/*
+	 * When invoking umount.crypt via the libtool helper script,
+	 * argv[0] is always "mount.crypt" due to the symlinking.
+	 */
+	if (strncmp(HX_basename(*argv), "umount", 6) == 0 ||
+	    getenv("PMT_DEBUG_UMOUNT") != NULL) {
 		struct umount_options opt;
 
 		memset(&opt, 0, sizeof(opt));
